unit sBevel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, sConst,
    sStyleUtil, sUtils;

type
  TsBevelShape = (sbsBox, sbsFrame, sbsTopLine, sbsBottomLine, sbsLeftLine,
    sbsRightLine, sbsSpacer{, sbsLabeled});

  TsBevel = class(TGraphicControl)//(TCustomControl)//(TGraphicControl)
  private
    FCaption : string;
    FsStyle : TsPaintStyle;
    FShape: TsBevelShape;
    FForbidMouse: boolean;
    procedure SetShape(Value: TsBevelShape);
    procedure WndProc (var Message: TMessage); override;
    procedure SetCaption(const Value: string);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property Anchors;
    property Caption : string read FCaption write SetCaption;
    property Constraints;
    property ForbidMouse: boolean read FForbidMouse write FForbidMouse default True;
    property ParentShowHint;
    property Shape: TsBevelShape read FShape write SetShape default sbsFrame;
    property ShowHint;
    property sStyle : TsPaintStyle read FsStyle write FsStyle;
    property Visible;
  end;

implementation

{ TsBevel }

constructor TsBevel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  sStyle := TsPaintStyle.Create(Self);
  sStyle.COC := COC_TsBevel;
  FForbidMouse := True;

  ControlStyle := ControlStyle + [csReplicatable];
  FShape := sbsFrame;
  Width := 50;
  Height := 50;
end;

procedure TsBevel.Paint;
const
  XorColor = $00FFD8CE;
var
  Color1, Color2: TColor;
  Temp: TColor;
  w: integer;

  procedure BevelRect(const R: TRect); begin
    Canvas.Pen.Color := Color1;
    Canvas.PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top), Point(R.Right, R.Top)]);
    Canvas.Pen.Color := Color2;
    Canvas.PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom), Point(R.Left, R.Bottom)]);
  end;

  procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer); begin
    Canvas.Pen.Color := C;
    Canvas.MoveTo(X1, Y1);
    Canvas.LineTo(X2, Y2);
  end;

begin
  if (csDesigning in ComponentState) then begin
    if (FShape = sbsSpacer) then begin
      Canvas.Pen.Style := psDot;
      Canvas.Pen.Mode := pmXor;
      Canvas.Pen.Color := XorColor;
      Canvas.Brush.Style := bsClear;
      Canvas.Rectangle(0, 0, ClientWidth, ClientHeight);
      Exit;
    end
    else begin
      Canvas.Pen.Style := psSolid;
      Canvas.Pen.Mode  := pmCopy;
      Canvas.Pen.Color := clBlack;
      Canvas.Brush.Style := bsSolid;
    end;
  end;

  Canvas.Pen.Width := 1;

  case sStyle.Painting.Bevel of
    bvLowered: begin
//      sStyle.PaintBorder(Canvas.Handle, 0, 0, Width, Height, False);
      Color1 := sStyle.Painting.ColorBorderBottom;
      Color2 := sStyle.Painting.ColorBorderTop;
    end;
    bvRaised: begin
      Color1 := sStyle.Painting.ColorBorderTop;
      Color2 := sStyle.Painting.ColorBorderBottom;
    end;
    bvSpace: begin
      Color1 := sStyle.Painting.ColorBorderBottom;
      Color2 := sStyle.Painting.ColorBorderBottom;
    end;
    bvNone: begin
      Exit;
    end;
  end;

  case FShape of
    sbsBox: BevelRect(Rect(0, 0, Width - 1, Height - 1));
{
    sbsLabeled: begin
      w := Canvas.TextWidth(FCaption);
      if Width < Canvas.TextWidth(FCaption) then Width := w + 10;
      sStyle.WriteText(Canvas, PChar(FCaption), Enabled, 0, 0, w, Height, DT_LEFT);

      BevelLine(Color1, w + 3, Height div 2, Width, Height div 2);
      BevelLine(Color2, w + 3, Height div 2 + 1, Width, Height div 2 + 1);
    end;
}
    sbsFrame: begin
      Temp := Color1;
      Color1 := Color2;
      BevelRect(Rect(1, 1, Width - 1, Height - 1));
      Color2 := Temp;
      Color1 := Temp;
      BevelRect(Rect(0, 0, Width - 2, Height - 2));
    end;
    sbsTopLine: begin
      BevelLine(Color1, 0, 0, Width, 0);
      BevelLine(Color2, 0, 1, Width, 1);
    end;
    sbsBottomLine: begin
      BevelLine(Color1, 0, Height - 2, Width, Height - 2);
      BevelLine(Color2, 0, Height - 1, Width, Height - 1);
    end;
    sbsLeftLine: begin
      BevelLine(Color1, 0, 0, 0, Height);
      BevelLine(Color2, 1, 0, 1, Height);
    end;
    sbsRightLine: begin
      BevelLine(Color1, Width - 2, 0, Width - 2, Height);
      BevelLine(Color2, Width - 1, 0, Width - 1, Height);
    end;
  end;
end;

procedure TsBevel.SetCaption(const Value: string);
begin
  if FCaption <> Value then begin
    FCaption := Value;
    Invalidate;
  end;
end;

procedure TsBevel.SetShape(Value: TsBevelShape);
begin
  if Value <> FShape then begin
    FShape := Value;
    Invalidate;
  end;
end;

procedure TsBevel.WndProc(var Message: TMessage);
begin
  sStyle.WndProc(Message);
  if Message.Result <> 1 then
    inherited;
end;

end.
