unit sCheckedControl;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov. All rights reserved.
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.2, 24-02-2002
*************************************************************************)

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, ExtCtrls, sStyleUtil, StdCtrls,
  sConst, sGraphUtils, sUtils;

type

  TsCheckedControl = class(TWinControl)
  private
    FAllowGrayed: Boolean;
    FLayout: TCheckAreaLayout;
    FShowFocus: Boolean;
    FCheckColor: TColor;
    FState: TCheckBoxState;
//    FDown: boolean;
    FPressed : boolean;
//    CheckArea : TRect;
    FOnChange : TNotifyEvent;
    procedure SetState(Value: TCheckBoxState);
    procedure SetLayout (Value: TCheckAreaLayout);
    procedure SetTransparent(const Value: Boolean);
    procedure WMEraseBkGND (var Message: TMessage); message WM_ERASEBKGND;
    procedure SetCheckColor(const Value: TColor);
    procedure SetShowFocus(const Value: Boolean);
  protected
    FsStyle : TsStyle;
    procedure SetChecked (Value: Boolean); virtual;
    function BeforeCheck(Value : boolean) : boolean; virtual;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure WMPaint (var Message: TWMPaint); message WM_PAINT;
//    procedure Paint; override;
    procedure WndProc (var Message: TMessage); override;
    procedure Toggle; virtual;
    property State: TCheckBoxState read FState write SetState default cbUnchecked;
    property AllowGrayed: Boolean read FAllowGrayed write FAllowGrayed default False;
  public
    FChecked: Boolean;
    FTransparent: Boolean;
    procedure PaintControl;
    procedure DrawCheckArea; dynamic;
    procedure DrawParent;
    procedure DrawCheckText;
    constructor Create (AOwner: TComponent); override;
    procedure Invalidate; override;
//    property Canvas;
    property Transparent: Boolean read FTransparent write SetTransparent default false;
  published
    property sStyle: TsStyle read FsStyle write FsStyle;
    property ShowFocus: Boolean read FShowFocus write SetShowFocus default false;
    property Caption;
    property Checked: Boolean read FChecked write SetChecked default false;
    property CheckColor : TColor read FCheckColor write SetCheckColor default clBlack;
    property Enabled;
    property Font;
    property Layout: TCheckAreaLayout read FLayout write SetLayout default calLeft;
    property ParentFont;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property Action;
    property Anchors;
    property BiDiMode write SetBidiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
  end;

procedure DrawEllipseBorder(Canvas: TCanvas; l, t, r, b : integer; ColorTop, ColorBottom : TColor);

implementation

uses sPanel;

procedure DrawEllipseBorder(Canvas: TCanvas; l, t, r, b : integer; ColorTop, ColorBottom : TColor);
begin
  Canvas.pen.Color := ColorTop;
  Canvas.Arc(L, T, R, B, R + 1, T + 1, L, B);

  Canvas.pen.Color := ColorBottom;
  Canvas.Arc(L, T, R, B, L - 1, B - 1, R, T);
end;

constructor TsCheckedControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  sStyle := TsStyle.Create(Self);
  sStyle.COC := COC_TsCheckedControl;
  ControlStyle := ControlStyle + [csOpaque];
  OnKeyDown := sStyle.onKeyDown;
  FCheckColor := clBlack;
  FLayout := calLeft;
  TabStop := True;
  Enabled := true;
  Visible := true;
  FTransparent := True;
  SetBounds(0, 0, 100, 17);
  FPressed := False;
end;

procedure TsCheckedControl.SetLayout(Value: TCheckAreaLayout);
begin
  if FLayout <> Value then begin
    FLayout := Value;
    Invalidate;
  end;
end;

procedure TsCheckedControl.SetChecked(Value: Boolean);
begin
  if FChecked <> Value then begin
    FChecked := BeforeCheck(Value);
    Invalidate;
  end;
end;

procedure TsCheckedControl.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if not FPressed and Enabled then begin
    if (Button = mbLeft) then begin
      FPressed := True;
      if not Focused then begin
        SetFocus;
      end;
//      if sStyle.Shadow.Enabled and not (csDesigning in ComponentState) then begin
//        Left := Left + sStyle.Shadow.Offset div 3;
//        Top := Top + sStyle.Shadow.Offset div 3;
//        sStyle.Shadow.FLabel.Visible := False;
//      end
    end else
    inherited;
  end;
end;

procedure TsCheckedControl.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled and FPressed then begin
    if not (csDesigning in ComponentState) then begin
//      if sStyle.Shadow.Enabled then begin
//        Left := Left - sStyle.Shadow.Offset div 3;
//        Top := Top - sStyle.Shadow.Offset div 3;
//        sStyle.Shadow.FLabel.Visible := True;
//      end;
      Checked := not Checked;
      if Assigned(FOnChange) then FOnChange(Self);
      Click;
    end;
    FPressed := False;
  end else
  inherited;
end;

procedure TsCheckedControl.DrawCheckArea;
var
  CheckArea: TRect;
begin
  case Layout of
    calLeft: begin
      CheckArea := Rect(ClientRect.Left, ClientRect.Top + 1, ClientRect.Left + 14, ClientRect.Top + 15);
    end;
    calRight: begin
      CheckArea := Rect(ClientRect.Right - 14, ClientRect.Top + 1, ClientRect.Right, ClientRect.Top + 15);
    end;
  end;

  sStyle.FCacheBmp.Canvas.pen.Style := psClear;
  sStyle.FCacheBmp.Canvas.brush.color := sStyle.GetActiveColor;
  sStyle.FCacheBmp.Canvas.brush.style := bsSolid;
  sStyle.FCacheBmp.Canvas.FillRect(CheckArea);

  sStyle.FCacheBmp.Canvas.pen.Style := psSolid;
  if Checked then begin
    if Enabled then begin
      sStyle.FCacheBmp.Canvas.pen.color := CheckColor;
      sStyle.FCacheBmp.Canvas.brush.color := CheckColor;
    end
    else begin
      sStyle.FCacheBmp.Canvas.brush.color := clBtnShadow;
      sStyle.FCacheBmp.Canvas.pen.color := clBtnShadow;
    end;
    // Checked
    sStyle.FCacheBmp.Canvas.Polygon([Point(CheckArea.Left + 2, CheckArea.Top + 7),
                    Point(CheckArea.Left + 5, CheckArea.Top + 10),
                    Point(CheckArea.Left + 6, CheckArea.Top + 10),
                    Point(CheckArea.Left + 11, CheckArea.Top + 5),
                    Point(CheckArea.Left + 11, CheckArea.Top + 3),
                    Point(CheckArea.Left + 6, CheckArea.Top + 8),
                    Point(CheckArea.Left + 5, CheckArea.Top + 8),
                    Point(CheckArea.Left + 4, CheckArea.Top + 7)])
  end;
  sStyle.PaintBorder(sStyle.FCacheBmp.Canvas.Handle, CheckArea.Left, CheckArea.Top, CheckArea.Right, CheckArea.Bottom, False);

//  sStyle.CopyFromCache(Canvas.Handle, 0, 0, Width, Height);
//  BitBlt(Canvas.Handle, 0, 0, sStyle.FCacheBmp.Width, sStyle.FCacheBmp.Height, sStyle.FCacheBmp.Canvas.Handle, 0, 0, SRCCOPY);
end;

procedure TsCheckedControl.DrawCheckText;
var
  rText: TRect;
  Fmt: UINT;
begin
  Fmt := 0;
  case FLayout of
    calLeft: begin
      rText := Rect(ClientRect.Left + 16, ClientRect.Top + 1, ClientRect.Right - 1, ClientRect.Bottom - 1);
      Fmt := DT_WORDBREAK or DT_LEFT;
    end;
    calRight: begin
      rText := Rect(ClientRect.Left + 1, ClientRect.Top + 1, ClientRect.Right - 16, ClientRect.Bottom - 1);
      Fmt := DT_WORDBREAK or DT_RIGHT;
    end;
  end;

  sStyle.FCacheBmp.Canvas.Brush.Style := bsClear;
  sStyle.FCacheBmp.Canvas.Font := Self.Font;
  if Enabled then begin
    DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), Length(Caption), rText, Fmt);
  end
  else begin
    OffsetRect(rText, 1, 1);
    sStyle.FCacheBmp.Canvas.Font.Color := clBtnHighlight;
    DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), Length(Caption), rText, Fmt);
    OffsetRect(rText, -1, -1);
    sStyle.FCacheBmp.Canvas.Font.Color := clBtnShadow;
    DrawText(sStyle.FCacheBmp.Canvas.Handle, PChar(Caption), Length(Caption), rText, Fmt);
  end;
  if Focused and ShowFocus then begin
    sStyle.FCacheBmp.Canvas.DrawFocusRect(rText);
  end;
end;
{
procedure TsCheckedControl.Paint;
begin
  if not sStyle.FDrawingStop then begin
    PaintControl;
  end;
end;
}
procedure TsCheckedControl.SetTransparent(const Value: Boolean);
begin
  if FTransparent <> Value then begin
    FTransparent := Value;
    Invalidate;
  end;
end;

procedure TsCheckedControl.Invalidate;
begin
//  if not sStyle.FDrawingStop then begin
    inherited;
//    PaintControl;
    sStyle.AlignShadow;
//  end;
end;

procedure TsCheckedControl.WndProc(var Message: TMessage);
begin
  sStyle.WndProc(Message);
  inherited;
end;

procedure TsCheckedControl.WMEraseBkGND(var Message: TMessage);
begin
  Message.Result := 1;
end;

function TsCheckedControl.BeforeCheck(Value : boolean): boolean;
begin
  Result := Value;
end;

procedure TsCheckedControl.SetCheckColor(const Value: TColor);
begin
  if FCheckColor <> Value then begin
    FCheckColor := Value;
    Invalidate
  end
end;

procedure TsCheckedControl.SetShowFocus(const Value: Boolean);
begin
  if FShowFocus <> Value then begin
    FShowFocus := Value;
    Invalidate
  end;
end;

procedure TsCheckedControl.SetState(Value: TCheckBoxState);
begin
  if FState <> Value then begin
    FState := Value;
  end;
end;

procedure TsCheckedControl.Toggle;
begin
  case State of
    cbUnchecked:
      if AllowGrayed then State := cbGrayed else State := cbChecked;
    cbChecked: State := cbUnchecked;
    cbGrayed: State := cbChecked;
  end;
end;

procedure TsCheckedControl.DrawParent;
var
  aRect: TRect;
begin
  aRect := GetClientRect;
  sStyle.FCacheBmp.Height := aRect.Bottom - aRect.Top + 1;
  sStyle.FCacheBmp.Width := aRect.Right - aRect.Left + 1;

  if sStyle.GetParentCache <> nil then begin
    FadeRect(sStyle.GetParentCache.Canvas, Rect(Left, Top, Left + Width, Top + Height), sStyle.FCacheBmp.Canvas, Point(0, 0), 100, clWhite, 0, ssRectangle);
  end
  else begin
//    sStyle.Painting.ParentColor := True;
    ParentColor := True;
    sStyle.FCacheBmp.Canvas.Pen.Style := psClear;
    sStyle.FCacheBmp.Canvas.Brush.Style := bsSolid;
    sStyle.FCacheBmp.Canvas.Brush.Color := Color;
    sStyle.FCacheBmp.Canvas.Rectangle(Rect(0, 0, sStyle.FCacheBmp.Width, sStyle.FCacheBmp.Height));
  end;
end;

procedure TsCheckedControl.PaintControl;
var
  sRect : TRect;
begin
  DrawParent;
  if sStyle.Shadow.Enabled and sStyle.Shadow.FLabel.Visible then begin
    case Layout of
      calLeft: begin
        sRect := Rect(ClientRect.Left,
                      ClientRect.Top + 1,
                      ClientRect.Left + 14,
                      ClientRect.Top + 15);
      end;
      calRight: begin
        sRect := Rect(ClientRect.Right - 14,
                      ClientRect.Top + 1,
                      ClientRect.Right,
                      ClientRect.Top + 15);
      end;
    end;
    OffsetRect(sRect, sStyle.Shadow.Offset, sStyle.Shadow.Offset);
    FadeRect(sStyle.FCacheBmp.Canvas, sRect,
                   sStyle.FCacheBmp.Canvas,
                   Point(sRect.Left, sRect.Top),
                   sStyle.Shadow.Transparency,
                   sStyle.Shadow.Color,
                   sStyle.Shadow.Blur, ssRectangle);
  end;
  DrawCheckText;
  DrawCheckArea;
end;

procedure TsCheckedControl.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if not FPressed and Enabled then begin
    if (Key = VK_SPACE) then begin
      FPressed := True;
//      if sStyle.Shadow.Enabled and not (csDesigning in ComponentState) then begin
//        sStyle.Shadow.FLabel.Visible := False;
//        Left := Left + sStyle.Shadow.Offset div 3;
//        Top := Top + sStyle.Shadow.Offset div 3;
//      end
    end else
    inherited;
  end;
end;

procedure TsCheckedControl.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_SPACE) and FPressed then begin
//    if sStyle.Shadow.Enabled and not (csDesigning in ComponentState) then begin
//      sStyle.Shadow.FLabel.Visible := True;
//      Left := Left - sStyle.Shadow.Offset div 3;
//      Top := Top - sStyle.Shadow.Offset div 3;
//    end;
    Checked := not Checked;
    if Assigned(FOnChange) then FOnChange(Self);
    FPressed := False;
  end
  else
  inherited;
end;

procedure TsCheckedControl.WMPaint(var Message: TWMPaint);
var
  SaveIndex: Integer;
  DC: HDC;
  PS: TPaintStruct;
  aRect : TRect;
begin
  if sStyle.Flat and not (csDestroying in ComponentState) and not (csLoading in ComponentState) then begin
    DC := Message.DC;
    if DC = 0 then DC := BeginPaint(Handle, PS);
    SaveIndex := SaveDC(DC);

    PaintControl;
//    sStyle.CopyFromCache(DC, 0, 0, Width, Height);

    if Message.DC = 0
      then aRect := PS.rcPaint
      else aRect := ClientRect;
    sStyle.CopyFromCache(DC, aRect.Left, aRect.Top, aRect.Right, aRect.Bottom);
    RestoreDC(DC, SaveIndex);
    if Message.DC = 0 then EndPaint(Handle, PS);
    PaintControls(DC, Self);
    Message.Result := 1;
  end;
end;

end.

