unit sColors;
(************************************************************************
sStyle components package 
Copyright (c) 2002 Serge Goncharov. All rights reserved.
                            Ukraine, Odessa
                            http://www.geocities.com/sstylish2002/
                            Serge@HyBce.com
 Version:	1.2, 24-02-2002
*************************************************************************)

interface

uses
  Classes, Controls, Graphics, Forms, sConst;


function RxIdentToColor(const Ident: string; var Color: Longint): Boolean;
function RxColorToString(Color: TColor): string;
function RxStringToColor(S: string): TColor;
procedure RxGetColorValues(Proc: TGetStrProc);


procedure Register;

implementation

uses
  Windows, SysUtils, DsgnIntf, Dialogs;


type
  TColorEntry = record
    Value: TColor;
    Name: PChar;
  end;

const
  clInfoBk16 = TColor($02E1FFFF);
  clNone16 = TColor($02FFFFFF);
  ColorCount = 27;
  Colors: array[0..ColorCount - 1] of TColorEntry = (
    (Value: scDarkRed       ; Name: 'scDarkRed'              ),
    (Value: scDarkOrange    ; Name: 'scDarkOrange'           ),
    (Value: scDarkYellow    ; Name: 'scDarkYellow'           ),
    (Value: scDarkGreen     ; Name: 'scDarkGreen'            ),
    (Value: scDarkSky       ; Name: 'scDarkSky'              ),
    (Value: scDarkBlue      ; Name: 'scDarkBlue'             ),
    (Value: scDarkViolet    ; Name: 'scDarkViolet'           ),
    (Value: scDarkBrown     ; Name: 'scDarkBrown'            ),
    (Value: scDarkKaki      ; Name: 'scDarkKaki'             ),

    (Value: scRed           ; Name: 'scRed'                  ),
    (Value: scOrange        ; Name: 'scOrange'               ),
    (Value: scYellow        ; Name: 'scYellow'               ),
    (Value: scGreen         ; Name: 'scGreen'                ),
    (Value: scSky           ; Name: 'scSky'                  ),
    (Value: scBlue          ; Name: 'scBlue'                 ),
    (Value: scViolet        ; Name: 'scViolet'               ),
    (Value: scBrown         ; Name: 'scBrown'                ),
    (Value: scKaki          ; Name: 'scKaki'                 ),

    (Value: scLightRed      ; Name: 'scLightRed'             ),
    (Value: scLightOrange   ; Name: 'scLightOrange'          ),
    (Value: scLightYellow   ; Name: 'scLightYellow'          ),
    (Value: scLightGreen    ; Name: 'scLightGreen'           ),
    (Value: scLightSky      ; Name: 'scLightSky'             ),
    (Value: scLightBlue     ; Name: 'scLightBlue'            ),
    (Value: scLightViolet   ; Name: 'scLightViolet'          ),
    (Value: scLightBrown    ; Name: 'scLightBrown'           ),
    (Value: scLightKaki     ; Name: 'scLightKaki'            )

  );

function RxColorToString(Color: TColor): string;
var
  I: Integer;
begin
  if not ColorToIdent(Color, Result) then begin
    for I := Low(Colors) to High(Colors) do
      if Colors[I].Value = Color then
      begin
        Result := StrPas(Colors[I].Name);
        Exit;
      end;
    FmtStr(Result, '$%.8x', [Color]);
  end;
end;

function RxIdentToColor(const Ident: string; var Color: Longint): Boolean;
var
  I: Integer;
  Text: array[0..63] of Char;
begin
  StrPLCopy(Text, Ident, SizeOf(Text) - 1);
  for I := Low(Colors) to High(Colors) do
    if StrIComp(Colors[I].Name, Text) = 0 then begin
      Color := Colors[I].Value;
      Result := True;
      Exit;
    end;
  Result := IdentToColor(Ident, Color);
end;

function RxStringToColor(S: string): TColor;
var
  I: Integer;
  Text: array[0..63] of Char;
begin
  StrPLCopy(Text, S, SizeOf(Text) - 1);
  for I := Low(Colors) to High(Colors) do
    if StrIComp(Colors[I].Name, Text) = 0 then
    begin
      Result := Colors[I].Value;
      Exit;
    end;
  Result := StringToColor(S);
end;

procedure RxGetColorValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  GetColorValues(Proc);
  for I := Low(Colors) to High(Colors) do Proc(StrPas(Colors[I].Name));
end;

{ TRxColorProperty }

type
  TRxColorProperty = class(TColorProperty)
  public
    function GetValue: string; override;
    procedure GetValues (Proc: TGetStrProc); override;
    procedure SetValue (const Value: string); override;
    procedure ListDrawValue(const Value: string; ACanvas: TCanvas;
      const ARect: TRect; ASelected: Boolean); override;
  end;

function TRxColorProperty.GetValue: string;
var
  Color: TColor;
begin
  try
  Color := TColor(GetOrdValue);
  if Color = clNone16 then Color := clNone
  else if Color = clInfoBk16 then Color := clInfoBk;
  Result := RxColorToString(Color);
  except
  end;
end;

procedure TRxColorProperty.GetValues(Proc: TGetStrProc);
begin
  try
  RxGetColorValues(Proc);
  except
  end;
end;

procedure TRxColorProperty.SetValue(const Value: string);
begin
  try
  SetOrdValue(RxStringToColor(Value));
  except
  end;
end;

procedure TRxColorProperty.ListDrawValue(const Value: string; ACanvas: TCanvas;
  const ARect: TRect; ASelected: Boolean);

  function ColorToBorderColor(AColor: TColor): TColor;
  type
    TColorQuad = record
      Red, Green, Blue, Alpha: Byte;
    end;
  begin
  try
    if (TColorQuad(AColor).Red > 192) or (TColorQuad(AColor).Green > 192) or
       (TColorQuad(AColor).Blue > 192) then
      Result := clBlack
    else if ASelected then
      Result := clWhite
    else
      Result := AColor;
  except
  end;
  end;

var
  vRight: Integer;
  vOldPenColor, vOldBrushColor: TColor;
begin
  vRight := (ARect.Bottom - ARect.Top) + ARect.Left;
  with ACanvas do
  try
    vOldPenColor := Pen.Color;
    vOldBrushColor := Brush.Color;
    Pen.Color := Brush.Color;
    Rectangle(ARect.Left, ARect.Top, vRight, ARect.Bottom);
    Brush.Color := RxStringToColor(Value);
    Pen.Color := ColorToBorderColor(ColorToRGB(Brush.Color));
    Rectangle(ARect.Left + 1, ARect.Top + 1, vRight - 1, ARect.Bottom - 1);
    Brush.Color := vOldBrushColor;
    Pen.Color := vOldPenColor;
  finally
    ACanvas.TextRect(Rect(vRight, ARect.Top, ARect.Right, ARect.Bottom),
      vRight + 1, ARect.Top + 1, Value);
  end;
end;

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TColor), TPersistent, '', TRxColorProperty);
end;

end.
